﻿using CarRepairShopContracts.BindingModels;
using CarRepairShopContracts.ViewModels;
using CarRepairShopDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace CarRepairShopFileImplement.Models
{
    internal class Shop : IShopModel
    {
        public int Id { get; set; }

        public string ShopName { get; set; } = string.Empty;

        public string ShopAddress { get; set; } = string.Empty;

        public DateTime DateOpen { get; set; }

        public int MaxCountRepairs { get; set; }

        public Dictionary<int, int> countRepair { get; private set; } = new();

        public Dictionary<int, (IRepairModel, int)> _repairs = null;

        public Dictionary<int, (IRepairModel, int)> ShopRepairs
        {
            get
            {
                if(_repairs == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _repairs = countRepair.ToDictionary(x => x.Key, y => ((source.Repairs.FirstOrDefault(z => z.Id == y.Key) as IRepairModel)!, y.Value));
                }
                return _repairs;
            }
        }

        public static Shop? Create(ShopBindingModel? model)
        {
            if (model == null) return null;
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                ShopAddress = model.ShopAddress,
                DateOpen = model.DateOpen,
                MaxCountRepairs = model.MaxCountRepairs,
                countRepair = model.ShopRepairs.ToDictionary(x => x.Key, x=> x.Value.Item2)
            };
        }
        public static Shop? Create(XElement element)

        {
            if (element == null) return null;
            return new Shop()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                ShopAddress = element.Element("ShopAddress")!.Value,
                DateOpen = Convert.ToDateTime(element.Element("DateOpen")!.Value),
                MaxCountRepairs = Convert.ToInt32(element.Element("MaxCountRepairs")!.Value),
                countRepair = element.Element("ShopRepairs")!.Elements("ShopRepairs").ToDictionary(x => Convert.ToInt32(x.Element("Key")?.Value), x => Convert.ToInt32(x.Element("Value")?.Value))
            };
        }

        public void Update(ShopBindingModel? model)
        {
            if (model == null) return;
            ShopName = model.ShopName;
            ShopAddress = model.ShopAddress;
            DateOpen = model.DateOpen;
            MaxCountRepairs = model.MaxCountRepairs;
            countRepair = model.ShopRepairs.ToDictionary(x => x.Key, x => x.Value.Item2);
            _repairs = null;
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            ShopAddress = ShopAddress,
            DateOpen = DateOpen,
            MaxCountRepairs = MaxCountRepairs,
            ShopRepairs = ShopRepairs
        };
        public XElement GetXElement => new("Shop",
            new XAttribute("Id", Id),
            new XElement("ShopName", ShopName),
            new XElement("ShopAddress", ShopAddress),
            new XElement("DateOpen", DateOpen.ToString()),
            new XElement("MaxCountRepairs", MaxCountRepairs.ToString()),
            new XElement("ShopRepairs", countRepair.Select(x => new XElement("ShopRepairs",
                new XElement("Key", x.Key),
                new XElement("Value", x.Value))).ToArray()));
    }
}
